if(!org) var org={};
if(!org.janek) org.janek={};

org.janek.identitychooser_preferences = new function() {
  var self = this;
  self.identitiesHelper = org.janek.identitychooser_identitieshelper();
  self.prefsHelper = org.janek.identitychooser_prefshelper();
  self.colorizeDisabledColor = "#EEEEEE";

  var pub = {};

  //
  // Initialize class
  //
  pub.init = function() {
    var preferencesNode =
      document.getElementById("identitychooser-prefs-preferences");

    var identitiesConfigListBox =
      document.getElementById("identitychooser-config-listbox");

    var allIdentities =
      self.identitiesHelper.getIdentitiesAccountListUserSorted();
    for(var i = 0; i < allIdentities.length; i++)
      {
        var identity = allIdentities[i].identity;

        // Create <preference>-tags for includeInMenu
        preferencesNode.appendChild(
          self.createIncludeInMenuPreference(identity));
        preferencesNode.appendChild(
          self.createWarnIfUsedPreference(identity));

        var identityNameListCell =
          self.createIdentityNameListCell(identity);
        var identityIncludeInMenuListCell =
          self.createIdentityIncludeInMenuListCell(identity);
        var identityColorPickerListCell =
          self.createIdentityColorPickerListCell(identity);
        var identityWarnIfUsedListCell =
          self.createWarnIfUsedListCell(identity);

        var listItem = document.createElement("listitem");
        listItem.setUserData("identity", identity.key, null);
        listItem.setAttribute("allowevents", "true");
        listItem.appendChild(identityNameListCell);
        listItem.appendChild(identityIncludeInMenuListCell);
        listItem.appendChild(identityColorPickerListCell);
        listItem.appendChild(identityWarnIfUsedListCell);
        identitiesConfigListBox.appendChild(listItem);
      }

    self.identitiesListBoxClicked();

    //
    // The identities's order, colors etc. are not modeled using
    // <preferences>, thus, requiring us to save the prefs ourselves,
    // which usually happens when the user closes the dialog via the
    // "OK" button. On Mac/Linux, though, this button does not exist
    // because instantApply is active. Therefore, when instantApply is
    // active, we need to trigger pref saving ourselves. To make it
    // easier, we save the prefs whenever the user clicks somewhere in
    // the tab pannel.
    //
    if(Application.prefs.get("browser.preferences.instantApply").value)
      {
        var identityConfigPanel =
          document.getElementById("identitychooser-identity-config-panel");

        identityConfigPanel.addEventListener("click",
                                             pub.onPrefWindowAccept,
                                             false);
      }


    identitiesConfigListBox.addEventListener("click",
                                             self.identitiesListBoxClicked,
                                             false);

    // Register event listener for shutdown
    window.addEventListener("unload", self.onUnload, false);
  };

  self.identitiesListBoxClicked = function() {
    var identitiesConfigListBox =
      document.getElementById("identitychooser-config-listbox");

    document.getElementById("move-up-button").disabled =
      (identitiesConfigListBox.selectedCount < 1);
    document.getElementById("move-down-button").disabled =
      (identitiesConfigListBox.selectedCount < 1);
  }

  self.createColorPreference = function(identity) {
    var colorPreference = document.createElement("preference");
    colorPreference.setAttribute(
      "id",
      self.prefsHelper.getFullColorPrefKey(identity));
    colorPreference.setAttribute(
      "name",
      self.prefsHelper.getFullColorPrefKey(identity));
    colorPreference.setAttribute("type",
                                 "string");

    return colorPreference;
  }

  self.createWarnIfUsedPreference = function(identity) {
    var preference = document.createElement("preference");
    preference.setAttribute(
      "id",
      self.prefsHelper.getFullWarnIfUsedPrefKey(identity));
    preference.setAttribute(
      "name",
      self.prefsHelper.getFullWarnIfUsedPrefKey(identity));
    preference.setAttribute("type",
                            "bool");

    return preference;
  }

  self.createIncludeInMenuPreference = function(identity) {
    var preference = document.createElement("preference");
    preference.setAttribute(
      "id",
      self.prefsHelper.getFullIncludeInMenuPrefKey(identity));
    preference.setAttribute(
      "name",
      self.prefsHelper.getFullIncludeInMenuPrefKey(identity));
    preference.setAttribute("type",
                            "bool");

    return preference;
  }

  self.createIdentityNameListCell = function(identity) {
    var l = document.createElement("listcell");

    l.setAttribute("class",
                   "identityName");
    l.setAttribute("label",
                   self.identitiesHelper.identityToString(identity));

    return l;
  }

  self.createIdentityIncludeInMenuListCell = function(identity) {
    var l = document.createElement("listcell");

    l.setAttribute("class",
                   "includeInMenu");

    var checkBox = document.createElement("checkbox");

    checkBox.setAttribute(
      "preference",
      self.prefsHelper.getFullIncludeInMenuPrefKey(identity));
    checkBox.setAttribute("checked",
                          self.prefsHelper.getPrefIncludeInMenu(identity));

    l.appendChild(checkBox);

    return l;
  }

  self.createIdentityColorPickerListCell = function(identity) {
    var l = document.createElement("listcell");
    l.setAttribute("class",
                   "color");

    var colorizeIdentityCheckBox = document.createElement("checkbox");
    colorizeIdentityCheckBox.setAttribute(
      "checked",
      self.prefsHelper.getPrefColorizeIdentity(identity));
    colorizeIdentityCheckBox.addEventListener(
      "CheckboxStateChange",
      self.enableColorPickerCheckBoxClicked,
      false);
    l.appendChild(colorizeIdentityCheckBox);

    var identityColorPicker = document.createElement("colorpicker");
    identityColorPicker.setAttribute("type", "button");
    if(colorizeIdentityCheckBox.getAttribute("checked") == "true")
      {
        identityColorPicker.setAttribute(
          "color",
          self.prefsHelper.getPrefColor(identity));
      }
    else
      {
        identityColorPicker.setAttribute("color",
                                         self.colorizeDisabledColor);
        identityColorPicker.setAttribute("disabled", true);
      }
    l.appendChild(identityColorPicker);

    return l;
  }

  self.enableColorPickerCheckBoxClicked = function(event, src) {
    var checkBoxState = event.currentTarget.checked;
    var colorPicker = event.currentTarget.nextSibling;

    colorPicker.disabled = !checkBoxState;
    if(colorPicker.disabled)
      {
        colorPicker.color = self.colorizeDisabledColor;
      }
    else
      {
        var identityKey =
          event.currentTarget.parentNode.parentNode.getUserData("identity");
        var identity = self.identitiesHelper.findIdentity(identityKey);
        colorPicker.color = self.prefsHelper.getPrefColor(identity);
      }
  }

  self.createWarnIfUsedListCell = function(identity) {
    var l = document.createElement("listcell");

    l.setAttribute("class",
                   "warnIfUsed");

    var vbox = document.createElement("box");
    vbox.setAttribute("id", "configurebox" + identity.key);
    var checkBox = document.createElement("checkbox");
    checkBox.setAttribute(
      "preference",
      self.prefsHelper.getFullWarnIfUsedPrefKey(identity));
    checkBox.setAttribute("checked",
                          self.prefsHelper.getPrefWarnIfUsed(identity));
    checkBox.addEventListener(
      "CheckboxStateChange",
      self.warnIfUsedCheckBoxClicked,
      false);

    var configureButton = document.createElement("button");
    configureButton.setAttribute("label", "...");
    configureButton.setAttribute("class", "configurebutton");
    configureButton.addEventListener("click",
                                     self.openWarnIfUsedOptionsDialog,
                                     false);
    if(self.prefsHelper.getPrefWarnIfUsed(identity) == false)
      {
        configureButton.setAttribute("disabled", "true");
      }

    vbox.appendChild(checkBox);
    vbox.appendChild(configureButton);

    l.appendChild(vbox);

    return l;
  }

  self.warnIfUsedCheckBoxClicked = function(event, src) {
    var checkBoxState = event.currentTarget.checked;
    var configureButton = event.currentTarget.nextSibling;

    configureButton.disabled = !checkBoxState;
  }

  self.openWarnIfUsedOptionsDialog = function(event, src) {
    if(event.currentTarget.disabled)
      {
        return;
      }

    var listItem = event.currentTarget.parentNode.parentNode.parentNode;
    var identityKey = listItem.getUserData("identity");
    var identity = self.identitiesHelper.findIdentity(identityKey);

    var warnIfUsedMode = listItem.getUserData("warnMode");
    if(warnIfUsedMode == null)
      {
        warnIfUsedMode = self.prefsHelper.getPrefWarnIfUsedMode(identity);
      }

    var params = {inWarnMode: warnIfUsedMode,
                  outWarnMode: null};
    window.openDialog("chrome://identitychooser/content/identitychooser-sendwarning-options.xul", "",
                      "chrome, dialog, modal, resizable=yes",
                      params).focus();

    listItem.setUserData("warnMode", params.outWarnMode, null);
  }

  self.onUnload = function() {
    window.removeEventListener("load",
                               org.janek.identitychooser_preferences.init,
                               false);
    window.removeEventListener("unload", self.onUnload, false);

    var identitiesConfigListBox =
      document.getElementById("identitychooser-config-listbox");
    identitiesConfigListBox.removeEventListener("click",
                                                self.identitiesListBoxClicked,
                                                false);
    if(Application.prefs.get("browser.preferences.instantApply").value)
      {
        var identityConfigPanel =
          document.getElementById("identitychooser-identity-config-panel");

        identityConfigPanel.removeEventListener("click",
                                                pub.onPrefWindowAccept,
                                                false);
      }
  }

  pub.moveSelectedIdentityUp = function() {
    var listBox =
      document.getElementById("identitychooser-config-listbox");

    if(listBox.selectedCount == 1)
      {
        var curSelectedIndex = listBox.selectedIndex;

        if(curSelectedIndex - 1 >= 0)
          {
            var curItem = listBox.selectedItem;
            var prevSibling = curItem.previousSibling;

            listBox.removeChild(curItem);
            listBox.insertBefore(curItem, prevSibling);
            listBox.selectedItem = curItem;
          }
      }
  }

  pub.moveSelectedIdentityDown = function() {
    var listBox =
      document.getElementById("identitychooser-config-listbox");

    if(listBox.selectedCount == 1)
      {
        var curSelectedIndex = listBox.selectedIndex;

        if(curSelectedIndex + 1 < listBox.itemCount)
          {
            var curItem = listBox.selectedItem;
            var nextSibling = curItem.nextSibling;

            listBox.removeChild(curItem);
            listBox.insertBefore(curItem, nextSibling.nextSibling);
            listBox.selectedItem = curItem;
          }
      }
  }

  pub.onPrefWindowAccept = function() {
    var listBox =
      document.getElementById("identitychooser-config-listbox");

    for(var i = 0; i < listBox.itemCount; i++)
      {
        var listItem = listBox.getItemAtIndex(i);
        var identityKey = listItem.getUserData("identity");
        var identity = {
          "key" : identityKey
        };

        self.prefsHelper.setMenuPosition(identity, i);

        var colorPicker = listItem.getElementsByTagName("colorpicker")[0];
        if(!colorPicker.disabled)
          {
            self.prefsHelper.setPrefColor(identity,
                                          colorPicker.color);
          }
        self.prefsHelper.setPrefColorizeIdentity(identity,
                                                 !colorPicker.disabled);

        var warnIfUsedMode = listItem.getUserData("warnMode");
        if(warnIfUsedMode != null)
          {
            self.prefsHelper.setPrefWarnIfUsedMode(identity,
                                                   warnIfUsedMode);
          }
      }

    return true;
  }

  return pub;
};


// Init class after windows loaded
window.addEventListener("load",
                        org.janek.identitychooser_preferences.init,
                        false);
